const BizResultCode = require('./BaseResultCode');
const logger = require('./logger')('OrderTimer.js')
const userService = require('./UserService')
const orderService = require('./OrderService')
const config = require('config');
const DMC = require('dmc.js');
class OrderTimer {
    static async buildMerkleTimer() {
        var orderList = await orderService.getOrdersUnCompleteMerkle();
        if (orderList instanceof BizResultCode) {
            return;
        }
        if (orderList.length == 0) {
            return;
        }

        for (const order of orderList) {
            var foggieId = order.foggie_id;
            var taskId = order.build_merkle_task_id;
            var orderId = order.order_id;
            var email = order.email;

            if (!foggieId || !taskId || !orderId || !email) {
                logger.error("param valid failed,orderId:{}", orderId);
            }

            var getMerkleEvent = {
                id: foggieId,
                taskId: taskId
            }

            var powClient = orderService.getPowGrpcClient();
            powClient.GetAsyncMerkleRoot(getMerkleEvent, async function (err, data) {
                if (err) {
                    logger.error('GetAsyncMerkleRoot failed, orderId{}, err:{}', orderId, err);
                    return;
                }
                var roots = data.roots;
                if (roots.length == 0) {
                    logger.error('Calculating merkle, orderId{}', orderId);
                    return;
                }
                // todo valid calculate merkle state
                // update order merkle state
                orderService.updateOrderCompleteMerkle(email, orderId);

                // valid challenge state
                // get user challenge count miner not response challenge count
                var challengeState = orderService.getChallengeCountByState(orderId, [3]);
                if (challengeState instanceof BizResultCode) {
                    logger.info('challengeState is null, orderId{}', orderId);
                    return;
                }

                if (challengeState > 0) {
                    logger.error('challengeState is not response, orderId{}', orderId);
                    return;
                }
                var root = roots[0];
                var merkleRootBytes = root.root.normalRoot;
                var merkleRoot = Buffer.from(merkleRootBytes).toString('hex');
                var dataBlockCount = root.root.totalBlocks;
                var merkleVersion = root.root.version;
                logger.info('start pushMerkle orderId{},merkleRoot{},merkleVersion:{},blockCount:{}', orderId, merkleRoot, merkleVersion, dataBlockCount);

                var userInfo = await userService.getUserInfo(email);
                if (userInfo instanceof BizResultCode) {
                    logger.info('userInfo is null');
                    return;
                }
                var username = userInfo.username;

                var privateKey = await userService.getPrivateKeyByEmail(email);
                if (privateKey instanceof BizResultCode) {
                    logger.info('private key is null');
                    return;
                }
                var chainId = orderService.getChainId();
                if (chainId instanceof BizResultCode) {
                    logger.info('chainId is null');
                    return;
                }
                var chainConfig = config.get('chainConfig');
                var httpEndpoint = chainConfig.get("httpEndpoint");
                var dmcClient = DMC({
                    chainId: chainId,
                    keyProvider: privateKey,
                    httpEndpoint: httpEndpoint,
                    logger: {
                        log: null,
                        error: null
                    }
                });

                dmcClient.transact({
                    actions: [{
                        account: "dmc.token",
                        name: 'addmerkle',
                        authorization: [
                            {
                                actor: username,
                                permission: 'active'
                            }
                        ],
                        data: {
                            sender: username,
                            order_id: orderId,
                            merkle_root: merkleRoot,
                            data_block_count: dataBlockCount
                        }
                    }]
                }, {
                    blocksBehind: 3,
                    expireSeconds: 30,
                }).then((result) => {
                    orderService.savePuskMerkleRecord(orderId, email, merkleRoot, merkleVersion, dataBlockCount, result.transaction_id, result.processed.block_num);
                }).catch((err) => {
                    logger.error('pushMerkle failed, orderId{},merkleRoot{},merkleVersion:{},blockCount:{},err:{}', orderId, merkleRoot, merkleVersion, dataBlockCount, err);
                })
            });
        }
    }
}

module.exports = OrderTimer